Installation
============

Installing the Modelverse is unnecessary, as it is mere Python code and doesn't use installation scripts.
All scripts which are generally useful are found in the 'scripts' directory, and are written in OS-independent Python code.

You will, however, need to install a dependency: the [SCCD compiler and runtime](https://msdl.uantwerpen.be/git/simon/SCCD).

Starting up the Modelverse
==========================

Starting up the Modelverse is easy: simply execute the `scripts/run_local_modelverse.py` script, with as parameter the port you want to use.
By default, port 8001 is used.

Communicating with the Modelverse
=================================

Now that the Modelverse is running, you will want to communicate with it!
To do this, you can use whatever tool you want, as long as it can send and receive XML/HTTPRequests.
For example, a mere internet browser can already communicate with the Modelverse, though not in the most user-friendly way.

A nicer way is through the Python prompt script `scripts/prompt.py`.
After that, it will print out all the output of the Modelverse, and send in all your queries directly to the Modelverse.

Python wrapper
==============

To automatically communicate with the Modelverse in a programmatic way, a Python wrapper is provided.
This wrapper is found in `wrappers/modelverse.py`, and provides Python functions that make the necessary Modelverse requests.
At the moment, not all functions are implemented in the wrapper yet.

Performance
===========

Performance of the Modelverse is currently rather low.
This is primarily caused by the reliance on the action language, which is an explicitly modelled (and interpreted) language.
Additionally, the Modelverse runs remotely, meaning that all requests have to pass over the network.
Even when this is executed on the same machine, this causes quite some overhead.

Additional documentation
========================

Some additional documentation can be found online in the [Modelverse techreport](http://msdl.cs.mcgill.ca/people/yentl/files/Modelverse.pdf), describing the internal workings of the Modelverse, as well as a brief introduction on how to use it.
There is also [in-depth documentation](https://msdl.uantwerpen.be/documentation/modelverse) describing how to use the Modelverse and its various languages.

Tests
=====

Running the tests is easy: simply execute `scripts/run_tests.py` in the main modelverse folder.
This will invoke the necessary build commands (to create bootstrapping code etc.) and call the tests for each individual aspect of the Modelverse.
Note that testing is done using py.test, which is the only dependency of the Modelverse (and only for tests, of course).

Using PyPy
----------

Since all scripts chain the invocation with the same interpreter as originally invoking the script, you will need to install py.test for PyPy.
Assuming that you already have PyPy installed, you can simply install py.test using these commands:
```sh
wget https://msdl.uantwerpen.be/files/get-pip.py
pypy get-pip.py --user
pypy -m pip install pytest --user
```

From then on, you can simply invoke all tests in PyPy using:
```sh
pypy scripts/run_tests.py
```

