import unittest
from modelverse_state.main import ModelverseState

class TestReadDictNode(unittest.TestCase):
    def setUp(self):
        self.mvs = ModelverseState()

    def test_read_dict_node_no_exists(self):
        l = self.mvs.read_dict_node(-1, "abc")
        assert l == None

    def test_read_dict_node_not_found_edge(self):
        a = self.mvs.create_node()
        b = self.mvs.create_node()
        c = self.mvs.create_edge(a, b)
        assert a != None
        assert b != None
        assert c != None

        # Passing data is not enforced, as the data will be interpreted if necessary
        l = self.mvs.read_dict_node(c, "abc")
        assert l == None

    def test_read_dict_node_no_primitive(self):
        a = self.mvs.create_node()
        assert a != None

        # Passing data is not enforced, as the data will be interpreted if necessary
        l = self.mvs.read_dict_node(a, a)
        assert l == None

    def test_read_dict_node_node_simple(self):
        a = self.mvs.create_node()
        b = self.mvs.create_node()
        c = self.mvs.create_node()
        d = self.mvs.create_edge(a, b)
        e = self.mvs.create_edge(d, c)
        assert a != None
        assert b != None
        assert c != None
        assert d != None
        assert e != None

        l = self.mvs.read_dict_node(a, c)
        assert l == b

    def test_read_dict_node_multi(self):
        a = self.mvs.create_node()
        b = self.mvs.create_node()
        c = self.mvs.create_node()
        d = self.mvs.create_edge(a, b)
        e = self.mvs.create_edge(d, c)
        assert a != None
        assert b != None
        assert c != None
        assert d != None
        assert e != None

        g = self.mvs.create_node()
        h = self.mvs.create_node()
        i = self.mvs.create_edge(a, g)
        j = self.mvs.create_edge(i, h)
        assert g != None
        assert h != None
        assert i != None
        assert j != None

        l = self.mvs.read_dict_node(a, c)
        assert l == b

        l = self.mvs.read_dict_node(a, h)
        assert l == g
